"use strict";

import {handleCollapseCardHeaderOnClick} from './common.js';
import {handleDropDownOnClick, displayDropdownBoxLabel} from './common.js';
import {hasInvalidCharacter, isNumeric, isEmptyField} from './common.js';
import {checkRequiredFieldInItem, checkInvalidCharacterInItem} from './common.js';
import {handleInputLabels, handleModalOnToggleById, toggleLoadingIcon} from './common.js';
import {convertDatetimeToLocaleString, getFilterDateInputValue, checkStatusColor} from './common.js';
import {getFileExtensionFromMimeType} from './common.js';
import {getHistoryTable} from './component/history_table.js';
import {getHistoryModal, renderCRAResponse, renderRepliedMessageResponse} from './component/history_modal.js';
import {renderReminderMessageResponse} from './component/history_modal.js';
import {getDetailTypeFromMessageType} from './model/business_model.js';

export const History = (store, actions, tabname) => {
  const inst = {};
  inst.store = store;
  inst.actions = actions;
  inst.tag = store.getState().tag;
  inst.store.subscribe((action) => {
    const {tab} = inst.store.getState();
    if (tabname && tab !== tabname) {
      return;
    }
    if (action.type === 'SET_RECORDS') {
      const {records} = inst.store.getState();
      const container = document.querySelector(`[data-${inst.tag}='content']`).querySelector(`[data-${inst.tag}-history='table']`);
      inst.table.buildTable(container, records);
    } else if (action.type === 'SET_TOTAL_PAGES') {
      const {totalPages, page, size} = inst.store.getState();
      const container = document.querySelector(`[data-${inst.tag}='content']`).querySelector(`[data-${inst.tag}-history='table']`);
      inst.table.buildPagination(container, page, size, totalPages);
    } else if (action.type === 'SET_DETAIL') {
      const {detail, detailType} = inst.store.getState();
      if (detailType === 'data-file') {
        inst.dataFileModal.buildModalContent(detail);
        inst.dataFileModal.buildModalFooter(detail);
      } else if (detailType === 'dcr') {
        inst.dcrModal.buildModalContent(detail);
        inst.dcrModal.buildModalFooter(detail);
        inst.buildReplyModal(detail.messageId);
      }
      // if (detailType) {
      //   inst.buildResolveModal(detail.messageId, detail.resolved);
      // }
    } else if (action.type === 'SET_RECORD_LOADING') {
      const {tab, recordLoading} = inst.store.getState();
      const table = document.querySelector(`[data-${inst.tag}-${tab}='table']`); 
      const tableLoading = table?.querySelector("[data-table='loading']");
      toggleLoadingIcon(tableLoading, recordLoading);
    } else if (action.type === 'SET_DETAIL_LOADING') {
      const {detailLoading, detailType} = inst.store.getState();
      let modalContainer;
      if (detailType === 'data-file') {
        modalContainer = inst.dataFileModal.getContainer();
      } else if (detailType === 'dcr') {
        modalContainer = inst.dcrModal.getContainer();
      }
      if (modalContainer) {
        toggleLoadingIcon(modalContainer, detailLoading);
      }
    }
  });
  inst.table = Object.create(getHistoryTable());
  inst.setTable = () => {
    inst.table.fillHistoryTableRow = function(cols, rec) {
      // Handle messageId
      const messageIdElement = document.createElement('a');
      const detailType = getDetailTypeFromMessageType(rec.messageType);
      messageIdElement.textContent = rec.messageId;
      if (detailType) {
        messageIdElement.classList.add("font-medium");
        messageIdElement.classList.add("text-blue-600");
        messageIdElement.classList.add("hover:underline");
        const modalOnToggleParams = {
          messageType: rec.messageType,
          messageId: rec.messageId,
        };
        if (detailType === 'data-file') {
          messageIdElement.addEventListener('click', () => inst.dataFileModal.handleHistoryModalOnToggle(modalOnToggleParams));
        } else if (detailType === 'dcr') {
          messageIdElement.addEventListener('click', () => inst.dcrModal.handleHistoryModalOnToggle(modalOnToggleParams));
        }
      }
      cols[0].appendChild(messageIdElement);

      const msgtypeSpan = document.getElementById(`${inst.tag}-history-span-template`).cloneNode(true).content;
      Array.from(msgtypeSpan.children).filter(c => c.getAttribute(`data-${inst.tag}-span`) !== detailType).forEach(elem => {
        elem.remove();
      });
      if (msgtypeSpan.children.length) {
        cols[1].appendChild(msgtypeSpan);
      } else {
        cols[1].textContent = rec.messageType;
      }
      cols[2].textContent = rec.source;
      cols[3].textContent = convertDatetimeToLocaleString(rec.createDatetime);
      cols[4].textContent = convertDatetimeToLocaleString(rec.lastUpdateDatetime);
      // Handle Status
      const statusSpan = document.createElement('span');
      const statusTextColor = checkStatusColor(rec.status);
      if (statusTextColor) {
        statusSpan.classList.add(`text-${statusTextColor}`);
      }
      statusSpan.innerHTML = rec.status;
      cols[5].appendChild(statusSpan);
      let resolvedIcon;
      if (rec.resolved) {
        resolvedIcon = document.getElementById("history-table-resolved-icon").cloneNode(true).content.firstElementChild;
        cols[5].appendChild(resolvedIcon);
      }
      cols[6].textContent = rec.messageRemark;
    };
    inst.table.setPage = function(page) {
      inst.store.dispatch(inst.actions.setPage(page))
    };
  };
  inst.buildFilter = () => {
    const queryParams = ['messageId', 'messageType', 'fileName', 'dcrReferenceId',
                         'status', 'createDatetimeFrom', 'createDatetimeTo'];
    const handleQueryInputOnChange = (name, target) => {
      let value;
      if (target.tagName === 'INPUT') {
        if (target.type === 'text') {
          value = target.value;
        } else if (target.type === 'checkbox') {
          value = target.checked;
        } else if (target.type === 'datetime-local' || target.type === 'date') {
          value = getFilterDateInputValue(name, target.value);
        }
      } else if (target.tagName === 'SELECT') {
        value = target.value;
      }
      const card = document.querySelector(`[data-${inst.tag}='content']`).querySelector(`[data-${inst.tag}-history='filter']`);
      const labelContainer = card.querySelector(`[data-${inst.tag}-filter='${name}']`);
      handleInputLabels(labelContainer, value, target.tagName);
      inst.store.dispatch(inst.actions.setQuery({[name]: value}));
    };
    const handleSearchButtonOnClick = () => {
      const {recordLoading} = inst.store.getState();
      if (recordLoading) {
        return;
      }
      if (!inst.isSearchButtonActive()) {
        return;
      }
      inst.store.dispatch(inst.actions.getRecords());
    };
    const handleClearButtonOnClick = () => {
      const {recordLoading} = inst.store.getState();
      if (recordLoading) {
        return;
      }
      queryParams.forEach(q => {
        const fieldContainer = card.querySelector(`[data-${inst.tag}-filter='${q}']`);
        const inputs = Array.from(fieldContainer.querySelectorAll("input"));
        if (inputs && inputs.length) {
          inputs.forEach(input => {
            if (input.type === 'checkbox') {
              input.checked = false;
            } else {
              input.value = null;
            }
          });
        }
        const select = fieldContainer.querySelector("select");
        if (select) {
          select.selectedIndex = -1;
        }
      });
      const boxLabels = card.querySelectorAll("[data-dropdown='box']")
      boxLabels.forEach(elem => {
        elem.querySelector('p').innerHTML = "";
      });
      inst.store.dispatch(inst.actions.clearQuery());
      inst.store.dispatch(inst.actions.getRecords());
    };
    const buildCollapseCard = (card) => {
      const header = card.querySelector("[data-collapsecard='header']");
      header.addEventListener('click', () => handleCollapseCardHeaderOnClick(card));
    };
    const renderFilterContent = (card) => {
      queryParams.filter(q => q !== 'status' && q !== 'messageType').forEach(q => {
        const fieldContainer = card.querySelector(`[data-${inst.tag}-filter='${q}']`);
        const input = fieldContainer?.querySelector("input");
        if (input) {
          input.addEventListener("change", (e) => handleQueryInputOnChange(q, e.target));
        }
        const select = fieldContainer?.querySelector("select");
        if (select) {
          select.selectedIndex = -1;
          select.addEventListener("change", (e) => handleQueryInputOnChange(q, e.target));
        }
      });
      const statusSelect = card.querySelector(`[data-${inst.tag}-filter='status']`);
      statusSelect.querySelector("[data-dropdown='box']").addEventListener('click', () => handleDropDownOnClick(statusSelect));
      statusSelect.querySelector("[data-dropdown='mask']").addEventListener('click', () => handleDropDownOnClick(statusSelect));
      const statusInputs = statusSelect.querySelector("[data-dropdown='container']").querySelectorAll('input');
      const statusTarget = () => {
        return {
          value: Array.from(statusInputs).filter(i => i.checked).map(o => o.value),
          tagName: 'SELECT', 
        };
      };
      statusInputs.forEach(input => {
        input.addEventListener('change', () => displayDropdownBoxLabel(statusSelect));
        input.addEventListener('change', () => handleQueryInputOnChange('status', statusTarget()));
      });

      const msgtypeSelect = card.querySelector(`[data-${inst.tag}-filter='messageType']`);
      msgtypeSelect.querySelector("[data-dropdown='box']").addEventListener('click', () => handleDropDownOnClick(msgtypeSelect));
      msgtypeSelect.querySelector("[data-dropdown='mask']").addEventListener('click', () => handleDropDownOnClick(msgtypeSelect));
      const msgtypeInputs = msgtypeSelect.querySelector("[data-dropdown='container']").querySelectorAll('input');
      const msgtypeTarget = () => {
        return {
          value: Array.from(msgtypeInputs).filter(i => i.checked).map(o => o.value),
          tagName: 'SELECT', 
        };
      };
      msgtypeInputs.forEach(input => {
        input.addEventListener('change', () => displayDropdownBoxLabel(msgtypeSelect));
        input.addEventListener('change', () => handleQueryInputOnChange('messageType', msgtypeTarget()));
      });
    };
    inst.isSearchButtonActive = () => {
      const {query} = store.getState();
      const checkInvalidCharacterFields = ["messageId", "fileName", "dcrReferenceId", "username", "messageRemark", "departmentCode"];
      let isActive = checkInvalidCharacterInItem(query, checkInvalidCharacterFields);
      return isActive;
    };
    const card = document.querySelector(`[data-${inst.tag}='content']`).querySelector(`[data-${inst.tag}-history='filter']`);
    buildCollapseCard(card);
    renderFilterContent(card);
    const searchButton = card.querySelector("#history-filter-search");
    searchButton.addEventListener("click", handleSearchButtonOnClick);
    const clearButton = card.querySelector("#history-filter-clear");
    clearButton.addEventListener("click", handleClearButtonOnClick);
  };
  inst.initialize = () => {
    const {tag} = inst.store.getState();
    inst.tag = tag;
    inst.buildFilter();
    inst.setTable();
    store.dispatch(inst.actions.getRecords());
  };
  // Table related method
  return inst;
};
